@@ -437,7 +437,7 @@ bmap_next_offset(
 	int		error;			/* error return value */
 	xfs_bmbt_irec_t got;			/* current extent value */
 	xfs_ifork_t	*ifp;			/* inode fork pointer */
-	xfs_extnum_t	idx;			/* last extent used */
+	struct xfs_iext_cursor	icur;
 
 	if (XFS_IFORK_FORMAT(ip, whichfork) != XFS_DINODE_FMT_BTREE &&
 	    XFS_IFORK_FORMAT(ip, whichfork) != XFS_DINODE_FMT_EXTENTS &&
@@ -452,7 +452,7 @@ bmap_next_offset(
 	    (error = -libxfs_iread_extents(tp, ip, whichfork)))
 		return error;
 	bno = *bnop + 1;
-	if (!libxfs_iext_lookup_extent(ip, ifp, bno, &idx, &got))
+	if (!libxfs_iext_lookup_extent(ip, ifp, bno, &icur, &got))
 		*bnop = NULLFILEOFF;
 	else
 		*bnop = got.br_startoff < bno ? bno : got.br_startoff;
@@ -524,7 +524,7 @@ mk_rbmino(xfs_mount_t *mp)
 	 */
 	ip->i_df.if_flags = XFS_IFEXTENTS;
 	ip->i_df.if_bytes = ip->i_df.if_real_bytes = 0;
-	ip->i_df.if_u1.if_extents = NULL;
+	ip->i_df.if_u1.if_root = NULL;
 
 	ip->i_d.di_size = mp->m_sb.sb_rbmblocks * mp->m_sb.sb_blocksize;
 
@@ -772,7 +772,7 @@ mk_rsumino(xfs_mount_t *mp)
 	 */
 	ip->i_df.if_flags = XFS_IFEXTENTS;
 	ip->i_df.if_bytes = ip->i_df.if_real_bytes = 0;
-	ip->i_df.if_u1.if_extents = NULL;
+	ip->i_df.if_u1.if_root = NULL;
 
 	ip->i_d.di_size = mp->m_rsumsize;
 
@@ -882,7 +882,7 @@ mk_root_dir(xfs_mount_t *mp)
 	 */
 	ip->i_df.if_flags = XFS_IFEXTENTS;
 	ip->i_df.if_bytes = ip->i_df.if_real_bytes = 0;
-	ip->i_df.if_u1.if_extents = NULL;
+	ip->i_df.if_u1.if_root = NULL;
 
 
 
@@ -1081,7 +1081,7 @@ mv_orphanage(
 	if ((err = -libxfs_iget(mp, NULL, ino, 0, &ino_p)))
 		do_error(_("%d - couldn't iget disconnected inode\n"), err);
 
-	xname.type = xfs_mode_to_ftype(VFS_I(ino_p)->i_mode);
+	xname.type = libxfs_mode_to_ftype(VFS_I(ino_p)->i_mode);
 
 	if (isa_dir)  {
 		irec = find_inode_rec(mp, XFS_INO_TO_AGNO(mp, orphanage_ino),
@@ -3125,7 +3125,7 @@ check_for_orphaned_inodes(
 
 static void
 traverse_function(
-	work_queue_t		*wq,
+	struct workqueue	*wq,
 	xfs_agnumber_t 		agno,
 	void			*arg)
 {
@@ -3154,7 +3154,7 @@ traverse_function(
 
 		for (i = 0; i < XFS_INODES_PER_CHUNK; i++)  {
 			if (inode_isadir(irec, i))
-				process_dir_inode(wq->mp, agno, irec, i);
+				process_dir_inode(wq->wq_ctx, agno, irec, i);
 		}
 	}
 	cleanup_inode_prefetch(pf_args);
